-- this plugin is used to dynamically update custom annotation .plugindata files
-- the list of cells is passed via chat commands of the format "0TMCD_r_v_h_action_data" where r=region, v=cell v index, h=cell h index, action='A' to add or 'D' to delete the annotation and data is an encoded data string to be added/updated/removed in the file
-- -- note, to 'update' an annotation you must delete the old and add the new since the encoded string has no other identifier and must match exactly

local importPath=string.gsub(getfenv(1)._.Name,"%.TMCust","").."."
--import (importPath.."Table");
import "Turbine";
import "Turbine.UI";
--import "Turbine.UI.Lotro";
import (importPath.."VindarPatch");

string.encode=function(str)
	-- replace all non-alpha characters with a % escaped hex code
	local ret=""
	for i,v in ipairs({str:byte(1,-1)}) do
		if (v < 48) or (v>57 and v<65) or (v>90 and v<97) or (v > 125) then
			ret = ret .. "%" .. string.sub("00"..string.format("%x",tonumber(v)),-2)
		else
			ret = ret .. string.char(v)
		end
	end
	return ret
end
string.decode=function(str)
	local ret=""
	while string.len(str)~=0 do
		local char=string.sub(str,1,1)
		if char~="%" then
			ret=ret..char
			str=string.sub(str,2,-1)
		else
			local char=string.char(tonumber("0x"..string.sub(str,2,3)))
			ret=ret..char
			str = string.sub(str,4,-1)
		end
	end
	return ret;
end
string.split=function(str,separator)
	local tmpArray={};
	local tmpElem;
	str=tostring(str);
	if string.find("^$()%.[]*+-?",separator,1,true)~=nil then separator="%"..separator end
	for tmpElem in string.gmatch(str,"[^"..separator.."]*") do
		table.insert(tmpArray,tmpElem);
	end
	local tmpIndex=1;
	while tmpIndex<=#tmpArray do
		if tmpIndex==#tmpArray and tmpArray[tmpIndex]=="" then
			table.remove(tmpArray,tmpIndex);
		else
			if tmpArray[tmpIndex]=="" and tmpArray[tmpIndex+1]~="" then
				table.remove(tmpArray,tmpIndex);
			else
				tmpIndex=tmpIndex+1;
			end
		end
	end
	return tmpArray;
end

local cmdCount=0
local parms;
cellList={}
cmdList={}
local cmds=Turbine.Shell.GetCommands();
if cmds~=nil and type(cmds)=="table" then
	table.sort(cmds,function(arg1,arg2)if arg1<arg2 then return(true) end end);
	for cmdIndex=0,#cmds do
		if cmds[cmdIndex]~=nil then
			if cmdIndex>1 and cmds[cmdIndex]>="1" then
				break
			else
				parms=string.match(cmds[cmdIndex],"0TMCD_(.*)");
				if parms~=nil then
					table.insert(cellList,string.split(parms,'_'))
				end
			end
		end
	end
end
_G.AnnotData={}
local region,row,col
for k,v in ipairs(cellList) do
	if region==nil or region~=v[1] or row~=v[2] or col~=v[3] then
		if region~=nil and custAnnot~=nil then
			-- save current annots
			PatchDataSave(Turbine.DataScope.Account, "TM_Cust_Annot_"..region.."_"..row.."_"..col,custAnnot);
		end
		region=v[1]
		row=v[2]
		col=v[3]
		custAnnot=PatchDataLoad(Turbine.DataScope.Account, "TM_Cust_Annot_"..region.."_"..row.."_"..col);
		if custAnnot==nil then custAnnot={} end
	end
-- load existing custom annots if any
	-- scan custAnnot looking for a cell that matches v[5]
	local index
	for i,d in ipairs(cellData) do
		if d==v[5] then
			index=i
			break
		end
	end
	if v[4]=='D' then
		if index~=nil then
			table.remove(custAnnot,index)
		end
	else	
		if index==nil then
			-- add a new annot
			table.insert(custAnnot,v[5])
		end
	end
end
if region~=nil and custAnnot~=nil then
	-- save current annots
	PatchDataSave(Turbine.DataScope.Account, "TM_Cust_Annot_"..region.."_"..row.."_"..col,custAnnot);
end
plugin.Unload = function(sender,args)
	-- remove the 'done' command
	Turbine.Shell.RemoveCommand(tmpCmd)
end
tmpCmd=Turbine.ShellCommand()
Turbine.Shell.AddCommand("0TMCC",tmpCmd) -- signal that we are done

